#!/bin/ksh

# Installation for MC interim fix
# RenLei 07-01-26

origin_path=`pwd`
patch_path=`dirname $0`
cd $patch_path
. ./patch_env.sh
patch_path_full=`pwd`

if [ -w ${patch_path_full} ] ; then
	backup=true
fi

ITM_HOME=

if [ ${#} = 1 -o ${#} = 2 ] ; then

	ITM_HOME=`echo ${1} | sed -e 's/\/*$//'`

	if [ ! -d ${ITM_HOME}/licenses ] ; then
		echo ====== Could not find directory ${ITM_HOME}/licenses ======
		ITM_HOME=
	fi

	if [ ${#} = 2 -a "$ITM_HOME" != "" ] ; then
		silent=true
	fi
fi

if [ "${ITM_HOME}" = "" ] ; then
	echo Please input your ITM path, for example /opt/IBM/ITM
	read ITM_HOME
	if [ "${ITM_HOME}" = "" ] ; then
		echo Then try to use the default one - /opt/IBM/ITM
		ITM_HOME=/opt/IBM/ITM
	fi
fi

while [ ! -d ${ITM_HOME}/licenses ] ; do
	echo Could not find directory ${ITM_HOME}/licenses
	echo Please input the where you installed ITM:
	read ITM_HOME
done

cd $ITM_HOME
ITM_HOME=`pwd`
cd $patch_path_full

# read version files for prerequeisite check
ver_files=`ls ${ITM_HOME}/registry/mc* ${ITM_HOME}/registry/mq* ${ITM_HOME}/registry/ic* ${ITM_HOME}/registry/qi* ${ITM_HOME}/registry/cf* 2>/dev/null`

for i in ${ver_files} ; do
	pre_level=`grep VRMF $i | awk '{print $3}' | sed -e 's/..$//'`
	if [ "$pre_level" != "$PREREQUISITE" ] ; then
		echo The version file $i shows this module is at level ${pre_level}, the prerequisite of this patch is ${PREREQUISITE}, install will stop
		echo please up the module to at least $PREREQUISITE level
		exit 0
	fi
done

if [ "$ITM_HOME" = "$patch_path_full" ] ; then
	echo Could not run patch_install in ITM_HOME \($ITM_HOME\). Please extract the tar file to another empty folder.
	echo program exit ...
	exit 0
fi

# if the patch folder is writable, prepare the backup folder
if [ "${backup}" = "true" ] ; then
	if [ -r cur_backup ] ; then
		suffix=`cat cur_backup`
		if [ ${#suffix} != 2 ] ; then
			suffix=
		fi
	fi
	
	echo $suffix > pre_backup
	
	if [ "$suffix" = "" ] ; then
		suffix=`ls | grep -v backup....* | grep "backup[0-9][0-9]" | sort`
		suffix=`echo $suffix | sed -e 's/.*backup//'`
	fi
	
	if [ "$suffix" = "" -o "$suffix" = "99" ] ; then
		suffix=00
	else
		suffix=`expr $suffix + 1`
		if [ ${#suffix} = 1 ] ; then
			suffix=0${suffix}
		fi
	fi
	
	echo $suffix > cur_backup
	
	backup_dir=backup${suffix}
	backup_script=patch_uninstall${suffix}
	
	rm -rf ${backup_dir}
	mkdir ${backup_dir}
	mkdir -p ${backup_dir}/registry
	
	mv pre_backup ${backup_dir}
fi

#Get the target system ITM VERSION information
ITM_VER=`${ITM_HOME}/bin/cinfo -i | grep "Installer Lvl" | awk -F: '{ print $3 }' | awk -F. '{ print $2 }'`

file_list=`cat file_list`
target_files=
operation_list=
for i in ${file_list} ; do
	patch_files=`echo $patch_files $i`

	dir_top=`echo $i | awk  -F/ '{print $1}'`
	i_cuthead=`echo ${dir_top} | sed -e 's/^[a-z]*[0-9][0-9]//'`
	if [ "${dir_top}" = "${i_cuthead}" ] ; then
		if [ -e ${ITM_HOME}/$i ] ; then
			target_files=`echo $target_files ${ITM_HOME}/$i`
			operation_list=`echo $operation_list OVERRIDE`
		else
			#added for defect 179931
			tems_tab=`echo $i | grep PLACEHOLDER`
			if [ -z "${tems_tab}" ] ; then
				target_files=`echo $target_files NULL`
				operation_list=`echo $operation_list NOT_FOUND`

			else
				#echo tems_tab ${tems_tab}
				placeholder_name=`echo $i | sed -e 's/\/PLACEHOLDER\//\/\*\//'`
				#echo placeholder_name ${placeholder_name}
				match=`ls ${ITM_HOME}/${placeholder_name} 2>/dev/null`	
				#echo match   ${match}
				if [ "$match" = "" ] ; then
					target_files=`echo $target_files NULL`
					operation_list=`echo $operation_list NOT_FOUND`
				else
					target_files=`echo $target_files ${match}`
					count=`echo "${match}" | wc -l`

					#process arch related multiple match scenario
					if [ $count -ge 2 ] ; then
						j=0
						while [ j -lt count ] ; do
							operation_list=`echo $operation_list OVERRIDE`
							if [ j -gt 0 ] ; then 
								patch_files=`echo $patch_files $i`
							fi
							j=`expr $j + 1`
						done
					else
						operation_list=`echo $operation_list OVERRIDE`
					fi	 
				fi
	
			fi
		fi		
	else	
		# special process for arch code
		wildcard=`echo $i |sed -e "s/${dir_top}/\*/"`	
		match=`ls ${ITM_HOME}/${wildcard} 2>/dev/null`
		if [ "$match" = "" ] ; then
			target_files=`echo $target_files NULL`
			operation_list=`echo $operation_list NOT_FOUND`
		else
			target_files=`echo $target_files ${match}`
			count=`echo "${match}" | wc -l`

			#process arch related multiple match scenario
			if [ $count -ge 2 ] ; then
				j=0
				while [ j -lt count ] ; do
					if [ j -gt 0 ] ; then 
						patch_files=`echo $patch_files $i`
					fi
					operation_list=`echo $operation_list OVERRIDE`
					j=`expr $j + 1`
				done
			else
				operation_list=`echo $operation_list OVERRIDE`
			fi	 
		fi
	fi
done

# back up old files
if [ "${backup}" = "true" ] ; then
	echo '#This file is generated by patch_install' > ${backup_script}
	echo '#Please do not modify' >> ${backup_script} 
	echo >> ${backup_script} 
	chmod +x  ${backup_script} 
	mkdir -p ${backup_dir}
	
	echo We are backing up your old files first
	if [ "$silent" = "" ] ; then
		echo press any key to continue ...
		read continue_install
	fi
	
	
	i=
	for i in ${target_files} ; do
		if [ "$i" = "NULL" ] ; then
			continue
		fi
		old_file=`echo $i | sed -e 's/\//_D_/g'`
		temp_folder=`echo ${ITM_HOME} | sed -e 's/\//_D_/g'`
		old_file=`echo $old_file | sed -e "s/${temp_folder}_D_//" | sed -e 's/_D_/\//g'`
		folder=`dirname $old_file`
		mkdir -p ${backup_dir}/$folder
		cp -pr $i ${backup_dir}/$old_file
		echo cp -pr $i ${backup_dir}/$old_file
		echo cp -pr ${backup_dir}/$old_file $i >>  ${backup_script}
	done
fi

# Start to install ...
echo We are going to install the interim fix to the directory ${ITM_HOME}

if [ "$silent" = "" ] ; then
	echo If you do not want that
	echo or IF this is not correct, press CTRL-C now
	read continue_install
fi


i=0
set -A target_array
for _i in ${target_files} ; do
	target_array[$i]=$_i
	i=`expr $i + 1`
done

i=0
set -A operation_array
for _i in ${operation_list} ; do
	operation_array[$i]=$_i
	i=`expr $i + 1`
done
new_locations=
if [ -f ${patch_path_full}/new_files ] ; then
	
	new_file_list=`cat ${patch_path_full}/new_files`
	for dependency in ${new_file_list} ; do

		if [ "$dependency" = "--" ] ; then
			new_locations=
			continue
		elif [ "$new_locations" = "" ] ; then
			k=0
			for file_name in $patch_files ; do
				if [ "$dependency" = "$file_name" ] ; then
					if [ "${operation_array[$k]}" = "NOT_FOUND" ] ; then
						new_locations=
						k=`expr $k + 1`
						continue;
					fi

					new_locations=`echo $new_locations ${target_array[$k]}`
					j=`expr $k + 1`
					if [ "${operation_array[$j]}" = "NOT_FOUND" ] ; then
						really_new=TRUE
						k=`expr $k + 1`
						break
					fi
				fi
				k=`expr $k + 1`
			done
		elif [ "$really_new" = "TRUE" ] ; then
			operation_array[$k]=SKIP
			k=`expr $k + 1`
			for new_location in $new_locations ; do
				new_patch_file=$dependency
				patch_files=`echo $patch_files $new_patch_file`
				new_target_path=`dirname $new_location`
				file_name=`basename $new_patch_file`
				new_target_file=${new_target_path}/${file_name}
				target_files=`echo $target_files $new_target_file`
				target_array[$i]=$new_target_file
				operation_list=`echo $operation_list NEW`
				operation_array[$i]=NEW
				i=`expr $i + 1`
			done
		fi
	done
fi
i=0
not_found=
for file_name in ${patch_files} ; do
	if [ "${operation_array[$i]}" = "NOT_FOUND" ] ; then
		not_found=`echo $not_found $file_name`
	elif [ "${operation_array[$i]}" != "SKIP" ] ; then
		cp -pr ${patch_path_full}/$file_name ${target_array[$i]}
		if [[ "${operation_array[$i]}" = "NEW" && "${backup}" = "true" ]] ; then
			echo rm -rf ${target_array[$i]} >> $backup_script
		fi
	fi
	i=`expr $i + 1`
done

#backup mc mq ic qi cf version files
if [ "${backup}" = "true" ] ; then
	cp -pr ${ITM_HOME}/registry/mc* ${backup_dir}/registry 2>/dev/null
	cp -pr ${ITM_HOME}/registry/mq* ${backup_dir}/registry 2>/dev/null
	cp -pr ${ITM_HOME}/registry/ic* ${backup_dir}/registry 2>/dev/null
	cp -pr ${ITM_HOME}/registry/qi* ${backup_dir}/registry 2>/dev/null
	cp -pr ${ITM_HOME}/registry/cf* ${backup_dir}/registry 2>/dev/null
	
	echo cp -pr ${backup_dir}/registry/*   ${ITM_HOME}/registry   >> ${backup_script}
fi

#update version files

for i in ${ver_files} ; do
	OLD_LEVEL=`echo $IFIX_LEVEL | sed -e 's/[0-9]$//'`
	cat $i | sed -e "/^VRMF/s/${OLD_LEVEL}./${IFIX_LEVEL}/" > /tmp/tmp_file
	mv /tmp/tmp_file $i
done

echo =============================================================
echo =============================================================
if [ "${not_found}" != "" ] ; then
	echo Below file\(s\) were not copied during this installation.
	echo Installer did not find those files in the directory. This might be correct if the related components were not installed in the environment.
	i=
	for i in ${not_found} ; do
		echo $i
	done
	echo =============================================================
	echo =============================================================
fi
echo Install complete

cd $origin_path
